define([
    "./_Class",
    "./surveyRuns",
    "dojo/Deferred",
    "dojo/_base/declare",
    "dojo/_base/json",
    "dojo/_base/lang",
    "dojo/_base/xhr",
    "dojo/date/stamp"
], function(_Class, surveyRuns, Deferred, declare, json, lang, xhr, stamp) {

    var Responses = declare([_Class],{
        _collection: 'responses',
        _type: 'Response',
        create: function() {
            var obj = {
                type: this._type,
                answers: {},
                surveyRunId: null
            };
            return obj;
        },
        _deserialize: function(obj) {
            if (obj._surveyRun) {
                obj._surveyRun = surveyRuns._doDeserialize(obj._surveyRun);
            }
            if (obj.publicationDate) {
                obj.publicationDate = stamp.fromISOString(obj.publicationDate);
            }
        },
        _serialize: function(obj) {
            if (obj._surveyRun) {
                obj._surveyRun = surveyRuns._doSerialize(obj._surveyRun);
            }
            if (obj.publicationDate) {
                obj.publicationDate = stamp.toISOString(obj.publicationDate);
            }
        },
        getWithSecret: function(id,secret) {
            var query = xhr.objectToQuery({secret:secret});
            return xhr('GET',{
                url: '/api/open/responses/' + id + '?' + query,
                handleAs: 'json',
                contentType: false
            }).then(lang.hitch(this,'_doDeserialize'),function(err){
                return new Deferred().reject(json.fromJson(err.responseText));
            });
        },
        postWithSecret: function(response,secret) {
            var query = xhr.objectToQuery({secret:secret});
            var body = json.toJson(this._doSerialize(response));
            return xhr('POST',{
                url: '/api/open/responses?' + query,
                handleAs: 'json',
                contentType: 'application/json',
                rawBody: body
            }).then(lang.hitch(this,'_doDeserialize'),function(err){
                return new Deferred().reject(json.fromJson(err.responseText));
            });
        },
        putWithSecret: function(response,secret) {
            var query = xhr.objectToQuery({secret:secret});
            var body = json.toJson(this._doSerialize(response));
            return xhr('PUT',{
                url: '/api/open/responses/' + this.getId(response) + '?' + query,
                handleAs: 'json',
                contentType: 'application/json',
                rawBody: body
            }).then(lang.hitch(this,'_doDeserialize'),function(err){
                return new Deferred().reject(json.fromJson(err.responseText));
            });
        },
        removeWithSecret: function(response,secret) {
            var query = xhr.objectToQuery({secret:secret});
            var rev = this.getRev(response);
            var body = json.toJson(this._doSerialize(response));
            var headers = {};
            if ( rev ) {
                headers['If-Match'] = '"'+rev+'"';
            }
            return xhr('DELETE',{
                url: '/api/open/responses/' + this.getId(response) + '?' + query,
                headers: headers,
                handleAs: 'json',
                contentType: 'application/json',
                rawBody: body
            });
        }
    });

    return new Responses();

});
