define([
    "../app/Content",
    "../app/Page",
    "../app/Router",
    "../lib/func",
    "../model/classes/responses",
    "../model/classes/surveyRuns",
    "../widgets/LineWithActionsWidget",
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/lang",
    "dojo/when",
    "require",
    "dojo/text!./templates/surveyRun.html"
], function(Content, Page, Router, func, responses, surveyRuns, LineWithActionsWidget, array, declare, event, lang, when, require, template) {
    return declare([Page],{
        contextRequire: require,
        templateString: template,
        surveyRun: null,
        startup: function() {
            if ( this._started ) { return; }
            this.inherited(arguments);
            this.surveyRunWidget.on("blur",lang.hitch(this,'_onPropChange'));
            if ( this.surveyRunId ) {
                this._loadSurveyRun();
            } else {
                throw "No valid uid or survey passed!";
            }
        },
        _loadSurveyRun: function() {
            when(surveyRuns.load(this.surveyRunId))
            .then(lang.hitch(this,function(surveyRun){
                this.surveyRun = surveyRun;
                this.refreshSurveyRun();
                this._loadResponses();
            }));
        },
        refreshSurveyRun: function() {
            this.titleNode.innerHTML = this.surveyRun.title || "";
            this.surveySummaryWidget.set('value',this.surveyRun.survey);
            this.surveyRunWidget.set('value',this.surveyRun);
            this._onPropChange();
        },
        _loadResponses: function() {
            responses.query({surveyRunId:surveyRuns.getId(this.surveyRun)})
            .then(lang.hitch(this,function(allResponses){
                array.forEach(allResponses, function(response){
                    var actions = {
                        view: {
                            callback: function(){},
                            properties: {
                                title: "View response"
                            }
                        }
                    };
                    if ( !response.publicationDate ) {
                        actions.remove = {
                            callback: function(){},
                            properties: {
                                title: "Remove response"
                            }
                        };
                    }
                    var w = new LineWithActionsWidget({
                        actions: actions
                    });
                    var rid = responses.getId(response);
                    w.set('title',this._link(this._buildResponseURL(response),rid),rid);
                    w.placeAt(this.responsesNode);
                }, this);
            }));
        },
        _onPropChange: function(e) {
            var surveyRun = this.surveyRunWidget.get('value');
            if ( surveyRun.mode === "open" ) {
                this.runURLNode.innerHTML =
                    this._link(this._buildGeneralURL(this.surveyRun));
            } else {
                this.runURLNode.innerHTML =
                    "No general URL. Add individual respondents below.";
            }
        },
        _buildGeneralURL: function(surveyRun) {
            return 'response.html#!/surveyRuns/'+surveyRuns.getId(surveyRun)+'!secret='+surveyRun.secret;
        },
        _buildResponseURL: function(response) {
            return 'response.html#!/responses/'+responses.getId(response)+'!secret='+response.secret;
        },
        _link: function(url,label) {
            return '<a target="_blank" href="'+url+'">'+(label || url)+'</a>';
        },
        _onSave: function(evt) {
            if ( this.surveyRunWidget.validate() ) {
                lang.mixin(this.surveyRun,this.surveyRunWidget.get('value'));

                surveyRuns.save(this.surveyRun)
                .then(function() { 
                    Router.go('/surveys');
                },function(err){
                    Content.notify(err);
                });
            }
            if ( evt ) { event.stop(evt); }
            return false;
        },
        _onDiscard: function(evt) {
            Router.go('/surveys');
            if ( evt ) { event.stop(evt); }
            return false;
        }
    });
});

